import numpy as np
import random
import math
import pyglet
from pyglet.gl import *

def prep_3iq(n,dx):
    A = np.zeros((n,),dtype=np.complex64)
    for k in range(0,n):
        q = 2*math.pi*k/(dx*n) if k<=n//2 else 2*math.pi*(k-n)/(dx*n)
        A[k] = 3*1j*q
    return A

def prep_iq3(n,dx):
    A = np.zeros((n,),dtype=np.complex64)
    for k in range(0,n):
        q = 2*math.pi*k/(dx*n) if k<=n//2 else 2*math.pi*(k-n)/(dx*n)
        A[k] = 1j*q*q*q
    return A

###
N = 256
L = 20
dx = L/N

OP_3iq = prep_3iq(N,dx)
OP_iq3 = prep_iq3(N,dx)

x = np.linspace(-L/2,L/2,N)
u = np.exp(-x*x)*12

t = 0
dt = 0.00001

HSCALE=4
VSCALE=10
H = 800
config = pyglet.gl.Config(double_buffer=True)
win = pyglet.window.Window(N*HSCALE,H,config=config,resizable=False)
win.set_caption('KdV Equation')

batch = pyglet.graphics.Batch()

shapes=[ ]
lines=[ ]
def init_plot():
    for j in range(-100,100,10):
        line = pyglet.shapes.Line(0, H/2 + VSCALE*j, N*HSCALE, H/2 + VSCALE*j, width=2, color=(40, 40, 70), batch=batch)
        shapes.append(line)
        str = '{0}'.format(j)
        label_y = pyglet.text.Label(str,font_name='Helvetica', color=(255,255,255,255), font_size=12, x=20, y=H/2+VSCALE*j-6, batch=batch)
        shapes.append(label_y)
        str = '{0}'.format(-L/2)
        label_x0 = pyglet.text.Label(str,font_name='Helvetica', color=(255,255,255,255), font_size=12, x=0, y=H/2-25, batch=batch)
        shapes.append(label_x0)
        str = '{0}'.format(L/2)
        label_x1 = pyglet.text.Label(str,font_name='Helvetica', color=(255,255,255,255), font_size=12, x=N*HSCALE-40, y=H/2-25, batch=batch)
        shapes.append(label_x1)
    for i in range(N):
        line = pyglet.shapes.Line(x=i*HSCALE, y=H/2, x2=(i+1)*HSCALE, y2=H/2, color=(255, 127, 30), width=4, batch=batch)
        lines.append(line)
        

def nextstep():
    global u,t
    U = np.fft.fft(u)    
    for cnt in range(500):
        U1 = U * np.exp(OP_iq3 * dt)
        K1 = - OP_3iq * np.fft.fft(u*u)
        U2 = U1 + K1 * dt/2
        u2 = np.fft.ifft(U2).real
        K2 = - OP_3iq * np.fft.fft(u2*u2)
        U = U1 + K2 * dt
        u = np.fft.ifft(U).real
        t = t + dt
    
@win.event
def on_draw():
    win.clear() 
    glClearColor(0,0,0,1)
    for i in range(N-1):
        line = lines[i]
        line.y = H/2+u[i]*VSCALE
        line.y2 = H/2+u[i+1]*VSCALE
    batch.draw()
    str = 'T={0:.3f}'.format(t)
    label_time = pyglet.text.Label(str,font_name='Helvetica', color=(255,255,255,255), font_size=12, x=10, y=10)
    label_time.draw()    

def update(deltat):
    nextstep()

init_plot()    
pyglet.clock.schedule_interval(update, 1/20.0)
pyglet.app.run()
